@@ -50,41 +50,35 @@ sub write_httpd_conf {
     my %args = (conf_file => 't/httpd.conf', @_);
     my $DIR = `pwd`; chomp $DIR;
 
-    # Apache2 tweaks
-    my $Port = 'Port';
-    my $AccessConfig = 'AccessConfig /dev/null';
-    my $ResourceConfig = 'ResourceConfig /dev/null';
-    my $ScoreBoardFile = 'ScoreBoardFile /dev/null';
-    if ($args{version} =~ m/^2\./) {
-	$Port = 'Listen';
-	$AccessConfig = '';
-	$ResourceConfig = '';
-	$ScoreBoardFile = '';
-    }
-
     local *CONF;
     open CONF, ">$args{conf_file}" or die "Can't create $args{conf_file}: $!";
     print CONF <<EOF;
 
-$Port $args{port}
+Port $args{port}
 User $args{user}
-Group "$args{group}"
+Group $args{group}
 ServerName localhost
 DocumentRoot $DIR/t
 
 $args{modules}
 
-ErrorLog $DIR/t/logs/error_log
-PidFile $DIR/t/logs/httpd.pid
-$AccessConfig
-$ResourceConfig
-LockFile $DIR/t/logs/httpd.lock
+ErrorLog $DIR/t/error_log
+PidFile $DIR/t/httpd.pid
+AccessConfig /dev/null
+ResourceConfig /dev/null
+LockFile $DIR/t/httpd.lock
 TypesConfig /dev/null
 TransferLog /dev/null
-$ScoreBoardFile
+ScoreBoardFile /dev/null
 
 AddType text/html .html
 
+# Look in ./blib/lib
+#PerlModule ExtUtils::testlib
+<Perl>
+ use lib "$DIR/blib/lib", "$DIR/t/lib";
+</Perl>
+
 $args{include}
 EOF
 
@@ -134,16 +128,14 @@ sub get_test_params {
 	}
     } until ($found);
     system "$Config{lns} $httpd t/httpd";
-    $conf{httpd} = $httpd;
 
     # Default: search for dynamic dependencies if mod_so is present, don't bother otherwise.
     my $default = (`t/httpd -l` =~ /mod_so\.c/ ? 'y' : 'n');
     if (lc _ask("Search existing config file for dynamic module dependencies?", $default) eq 'y') {
-	my %compiled = $pkg->get_compilation_params('t/httpd');
+	my %compiled = $pkg->_get_compilation_params('t/httpd');
 
-	$conf{version} = $compiled{SERVER_VERSION};
-	$conf{config_file} = _ask("  Config file", $compiled{SERVER_CONFIG_FILE}, 1);
-	$conf{modules} = $pkg->_read_existing_conf($conf{config_file});
+	my $file = _ask("  Config file", $compiled{SERVER_CONFIG_FILE}, 1);
+	$conf{modules} = $pkg->_read_existing_conf($file);
     }
 
     # Get default user (apache doesn't like to run as root, special-case it)
@@ -158,7 +150,7 @@ sub get_test_params {
     return %conf;
 }
 
-sub get_compilation_params {
+sub _get_compilation_params {
     my ($self, $httpd) = @_;
 
     my %compiled;
@@ -166,9 +158,6 @@ sub get_compilation_params {
 	if (/([\w]+)="(.*)"/) {
 	    $compiled{$1} = $2;
 	}
-	if (/Server version: .*?([\d\.]+)/i) {
-	    $compiled{SERVER_VERSION} = $1;
-	}
     }
     $compiled{SERVER_CONFIG_FILE} =~ s,^,$compiled{HTTPD_ROOT}/,
 	unless $compiled{SERVER_CONFIG_FILE} =~ m,^/,;
@@ -185,39 +174,43 @@ sub _read_existing_conf {
     my @lines = grep {!m/^\s*\#/} <SERVER_CONF>;
     close SERVER_CONF;
 
+    my @includes;
+    foreach my $include (grep /^\s*Include\s+\S+/, @lines) {
+	my ($file) = $include =~ /^\s*Include\s+(\S+)/;
+	$file =~ s/^"//;
+	$file =~ s/"//;
+	push @includes, $file;
+	warn "ADDED INC $file\n";
+    }
+
+    my @modules       =   grep /^\s*(Add|Load)Module/, @lines;
+
     my ($server_root) = (map /^\s*ServerRoot\s*(\S+)/, @lines);
     $server_root =~ s/^"//;
     $server_root =~ s/"$//;
 
     $server_root ||= $default_root;
 
-    my @includes;
-    foreach my $include (grep /^\s*Include\s+\S+/, @lines) {
-	my ($file) = $include =~ /^\s*Include\s+(\S+)/;
-	$file =~ s/^"//;
-	$file =~ s/"//;
-	$file =~ s!^([^/])!$server_root/$1!;  # absolute path
-
-	if ($file =~ m/\*/) {
-	    # expand wildcard includes (used in Fedora Core 1 default config)
-	    my @add = glob $file;
-	    unless ($Apache::test::quiet) {
-		warn "expanding wildcard Include $file\n";
-		warn "ADDED INC $_\n" for @add;
-	    }
-	    push @includes, @add;
-	} else {
-	    push @includes, $file;
-	    warn "ADDED INC $file\n" unless $Apache::test::quiet;
-	}
-    }
-
-    my @modules = grep /^\s*(Add|Load|Clear)Module/, @lines;
-
     # Rewrite all modules to load from an absolute path.
     foreach (@modules) {
 	s!(\s)([^/\s]\S+/)!$1$server_root/$2!;
     }
+    # And do the same for includes.
+    foreach (@includes) {
+	s!^([^/])!$server_root/$1!;
+	warn "$_\n";
+    }
+
+    my $static_mods = $self->static_modules('t/httpd');
+    
+    my @load;
+    # Have to make sure that dir, autoindex and perl are loaded.
+    foreach my $module (qw(dir autoindex perl)) {
+       unless ($static_mods->{"mod_$module"} or grep /$module/i, @modules) {
+           warn "Will attempt to load mod_$module dynamically.\n";
+	    push @load, $module;
+	}
+    }
 
     # Follow each include recursively to find needed modules
     foreach my $include (@includes) {
@@ -226,17 +219,6 @@ sub _read_existing_conf {
     # The last bits only need to be done once.
     return @modules if $is_include;
 
-    my $static_mods = $self->static_modules('t/httpd');
-    
-    my @load;
-    # Have to make sure that dir, autoindex and perl are loaded.
-    foreach my $module (qw(dir autoindex perl)) {
-       unless ($static_mods->{"mod_$module"} or grep /$module/i, @modules) {
-           warn "Will attempt to load mod_$module dynamically.\n" unless $Apache::test::quiet;
-	    push @load, $module;
-	}
-    }
-
     # Directories where apache DSOs live.
     my @module_dirs = map {m,(/\S*)/,} @modules;
     
@@ -250,14 +232,12 @@ sub _read_existing_conf {
                }
 	    }
 	}
-       warn "Warning: couldn't find anything to load for 'mod_$module'.\n" unless $Apache::test::quiet;
+       warn "Warning: couldn't find anything to load for 'mod_$module'.\n";
     }
 
-    unless ($Apache::test::quiet) {
-	print "Adding the following dynamic config lines: \n";
-	print join '', @modules;
-	print "\n\n";
-    }
+    print "Adding the following dynamic config lines: \n";
+    print join '', @modules;
+    print "\n\n";
     return join '', @modules;
 }
 
@@ -275,14 +255,9 @@ sub _find_mod_perl_httpd {
 
     return $ENV{'APACHE'} if $ENV{'APACHE'} && $respect_env;
 
-    local $Apache::test::quiet = 1;
     foreach ( '/usr/local/apache/bin/httpd',
 	      '/usr/local/apache_mp/bin/httpd',
-	      '/usr/local/apache2/bin/httpd',
 	      '/opt/apache/bin/httpd',
-	      '/usr/sbin/apache-perl',
-	      '/usr/sbin/apache',
-	      '/usr/sbin/httpd',
 	      $self->_which('httpd'),
 	      $self->_which('apache'),
 	    ) {
@@ -295,15 +270,14 @@ sub _httpd_has_mod_perl {
 
     return 1 if `$httpd -l` =~ /mod_perl\.c/;
 
-    my %compiled = $self->get_compilation_params($httpd);
-
-    if ($compiled{SERVER_VERSION} =~ m/^2\./) {
-	warn("Apache $compiled{SERVER_VERSION} detected. Report problems to mason-users\@lists.sourceforge.net\n") unless $Apache::test::quiet;
-    }
+    my %compiled = $self->_get_compilation_params($httpd);
 
     if ($compiled{SERVER_CONFIG_FILE}) {
-	local $Apache::test::quiet = 1;
-	my @lines = $self->_read_existing_conf($compiled{SERVER_CONFIG_FILE},$compiled{HTTPD_ROOT});
+	local *SERVER_CONF;
+	open SERVER_CONF, $compiled{SERVER_CONFIG_FILE} or die "Couldn't open $compiled{SERVER_CONFIG_FILE}: $!";
+	my @lines = grep {!m/^\s*\#/} <SERVER_CONF>;
+	close SERVER_CONF;
+
 	return 1 if grep { /mod_perl/ } grep /^\s*(Add|Load)Module/, @lines;
     }
 
@@ -360,16 +334,31 @@ sub simple_fetch {
     $response->is_success;
 }
 
+#even if eval $mod fails, the .pm ends up in %INC
+#so the next eval $mod succeeds, when it shouldnot
+
+my %really_have = (
+   'Apache::Table' => sub { 
+       if ($ENV{MOD_PERL}) {
+	   return Apache::Table->can('TIEHASH');
+       }
+       else {
+	   return $net::callback_hooks{PERL_TABLE_API};
+       }
+   },
+);
+
+for (qw(Apache::Cookie Apache::Request)) {
+    $really_have{$_} = $really_have{'Apache::Table'};
+}
+
 sub have_module {
     my $mod = shift;
     my $v = shift;
     eval {# surpress "can't boostrap" warnings
-	local $SIG{__WARN__} = sub {};
-	if ($mod_perl2::VERSION >= 2.00) {
-	    # use Apache2 is no longer needed
-	} else {
-	    require Apache;
-	}
+	 local $SIG{__WARN__} = sub {};
+	 require Apache;
+	 require Apache::Constants;
     };
 
     eval "require $mod";
@@ -393,6 +382,9 @@ sub have_module {
 	warn "$@\n";
     }
 
+    if (my $cv = $really_have{$mod}) {
+	return 0 unless $cv->();
+    }
 
     print "module $mod is installed\n" unless $ENV{MOD_PERL};
     
@@ -471,10 +463,10 @@ test :: pure_all start_httpd run_tests   kill_httpd
 testdb:	start_httpd run_testsdb kill_httpd
 
 kill_httpd:
-	kill `cat t/logs/httpd.pid`
+	kill `cat t/httpd.pid`
 
 start_httpd:
-	t/httpd -f `pwd`/t/conf/httpd.conf
+	t/httpd -f `pwd`/t/httpd.conf
 
 run_tests :: pure_all
 	PERL_DL_NONLAZY=1 PORT=$conf{port}
